part of masamune_signin_twitter;

/// Sign in to Firebase using Twitter Login.
class TwitterAuth {
  const TwitterAuth._();

  static late final String _twitterAPIKey;
  static late final String _twitterAPISecret;
  static late final String _urlSchemeId;

  /// Set options for authentication.
  ///
  /// [twitterAPIKey]: Twitter API Key.
  /// [twitterAPISecret]: Twitter API Secret.
  static void initialize({
    required String twitterAPIKey,
    required String twitterAPISecret,
    required String urlSchemeId,
  }) {
    _twitterAPIKey = twitterAPIKey;
    _twitterAPISecret = twitterAPISecret;
    _urlSchemeId = urlSchemeId;
  }

  /// Gets the options for the provider.
  static const AuthProviderOptions options = AuthProviderOptions(
    id: "twitter",
    provider: _provider,
    title: "Twitter SignIn",
    text: "Sign in with your Twitter account.",
  );
  static Future<FirebaseAuthModel> _provider(
    BuildContext context,
    Duration timeout,
  ) {
    return signIn(timeout: timeout);
  }

  /// Sign in to Firebase using Twitter Login.
  ///
  /// [protorol]: Protocol specification.
  /// [timeout]: Timeout time.
  static Future<FirebaseAuthModel> signIn({
    Duration timeout = const Duration(seconds: 60),
  }) async {
    if (_twitterAPIKey.isEmpty ||
        _twitterAPISecret.isEmpty ||
        _urlSchemeId.isEmpty) {
      throw Exception(
          "There is no API key or other information. Please initialize it with [initialize].");
    }
    final auth = read(firebaseAuthProvider);
    await auth.signInWithProvider(
      providerCallback: (timeout) async {
        final twitter = TwitterLogin(
          apiKey: _twitterAPIKey,
          apiSecretKey: _twitterAPISecret,
          redirectURI: _urlSchemeId,
        );
        final result = await twitter.login();
        switch (result.status) {
          case TwitterLoginStatus.cancelledByUser:
            throw Exception("Login canceled");
          case TwitterLoginStatus.error:
            throw Exception(
                "Login terminated with error: ${result.errorMessage}");
          default:
            break;
        }
        if (result.authToken.isEmpty || result.authTokenSecret.isEmpty) {
          throw Exception("Login terminated with error: Token is empty.");
        }
        return TwitterAuthProvider.credential(
          accessToken: result.authToken!,
          secret: result.authTokenSecret!,
        );
      },
      providerId: TwitterAuthProvider.PROVIDER_ID,
      timeout: timeout,
    );
    return auth;
  }
}
